# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.sessions import keyboard_type_params, keyboard_shortcut_params
from ...types.sessions.keyboard_type_response import KeyboardTypeResponse
from ...types.sessions.keyboard_shortcut_response import KeyboardShortcutResponse

__all__ = ["KeyboardResource", "AsyncKeyboardResource"]


class KeyboardResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> KeyboardResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return KeyboardResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KeyboardResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return KeyboardResourceWithStreamingResponse(self)

    def shortcut(
        self,
        session_id: str,
        *,
        keys: SequenceNotStr[str],
        hold_time: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KeyboardShortcutResponse:
        """
        Performs a keyboard shortcut using the specified keys

        Args:
          keys: Array of keys to press simultaneously

          hold_time: Time to hold the keys down in milliseconds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/keyboard/shortcut",
            body=maybe_transform(
                {
                    "keys": keys,
                    "hold_time": hold_time,
                },
                keyboard_shortcut_params.KeyboardShortcutParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyboardShortcutResponse,
        )

    def type(
        self,
        session_id: str,
        *,
        text: str,
        delay: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KeyboardTypeResponse:
        """
        Types the specified text with optional delay between keystrokes

        Args:
          text: Text to type

          delay: Delay between keystrokes in milliseconds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/keyboard/type",
            body=maybe_transform(
                {
                    "text": text,
                    "delay": delay,
                },
                keyboard_type_params.KeyboardTypeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyboardTypeResponse,
        )


class AsyncKeyboardResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncKeyboardResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncKeyboardResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKeyboardResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncKeyboardResourceWithStreamingResponse(self)

    async def shortcut(
        self,
        session_id: str,
        *,
        keys: SequenceNotStr[str],
        hold_time: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KeyboardShortcutResponse:
        """
        Performs a keyboard shortcut using the specified keys

        Args:
          keys: Array of keys to press simultaneously

          hold_time: Time to hold the keys down in milliseconds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/keyboard/shortcut",
            body=await async_maybe_transform(
                {
                    "keys": keys,
                    "hold_time": hold_time,
                },
                keyboard_shortcut_params.KeyboardShortcutParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyboardShortcutResponse,
        )

    async def type(
        self,
        session_id: str,
        *,
        text: str,
        delay: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KeyboardTypeResponse:
        """
        Types the specified text with optional delay between keystrokes

        Args:
          text: Text to type

          delay: Delay between keystrokes in milliseconds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/keyboard/type",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "delay": delay,
                },
                keyboard_type_params.KeyboardTypeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyboardTypeResponse,
        )


class KeyboardResourceWithRawResponse:
    def __init__(self, keyboard: KeyboardResource) -> None:
        self._keyboard = keyboard

        self.shortcut = to_raw_response_wrapper(
            keyboard.shortcut,
        )
        self.type = to_raw_response_wrapper(
            keyboard.type,
        )


class AsyncKeyboardResourceWithRawResponse:
    def __init__(self, keyboard: AsyncKeyboardResource) -> None:
        self._keyboard = keyboard

        self.shortcut = async_to_raw_response_wrapper(
            keyboard.shortcut,
        )
        self.type = async_to_raw_response_wrapper(
            keyboard.type,
        )


class KeyboardResourceWithStreamingResponse:
    def __init__(self, keyboard: KeyboardResource) -> None:
        self._keyboard = keyboard

        self.shortcut = to_streamed_response_wrapper(
            keyboard.shortcut,
        )
        self.type = to_streamed_response_wrapper(
            keyboard.type,
        )


class AsyncKeyboardResourceWithStreamingResponse:
    def __init__(self, keyboard: AsyncKeyboardResource) -> None:
        self._keyboard = keyboard

        self.shortcut = async_to_streamed_response_wrapper(
            keyboard.shortcut,
        )
        self.type = async_to_streamed_response_wrapper(
            keyboard.type,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.shared.success_response import SuccessResponse
from ...types.sessions.all_status_response import AllStatusResponse

__all__ = ["AllResource", "AsyncAllResource"]


class AllResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AllResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AllResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AllResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AllResourceWithStreamingResponse(self)

    def delete(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuccessResponse:
        """Terminates all active browser sessions associated with the provided API key."""
        return self._delete(
            "/v1/sessions/all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponse,
        )

    def status(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AllStatusResponse:
        """
        Retrieves status information for all browser sessions associated with the API
        key.
        """
        return self._get(
            "/v1/sessions/all/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AllStatusResponse,
        )


class AsyncAllResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAllResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncAllResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAllResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncAllResourceWithStreamingResponse(self)

    async def delete(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuccessResponse:
        """Terminates all active browser sessions associated with the provided API key."""
        return await self._delete(
            "/v1/sessions/all",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponse,
        )

    async def status(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AllStatusResponse:
        """
        Retrieves status information for all browser sessions associated with the API
        key.
        """
        return await self._get(
            "/v1/sessions/all/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AllStatusResponse,
        )


class AllResourceWithRawResponse:
    def __init__(self, all: AllResource) -> None:
        self._all = all

        self.delete = to_raw_response_wrapper(
            all.delete,
        )
        self.status = to_raw_response_wrapper(
            all.status,
        )


class AsyncAllResourceWithRawResponse:
    def __init__(self, all: AsyncAllResource) -> None:
        self._all = all

        self.delete = async_to_raw_response_wrapper(
            all.delete,
        )
        self.status = async_to_raw_response_wrapper(
            all.status,
        )


class AllResourceWithStreamingResponse:
    def __init__(self, all: AllResource) -> None:
        self._all = all

        self.delete = to_streamed_response_wrapper(
            all.delete,
        )
        self.status = to_streamed_response_wrapper(
            all.status,
        )


class AsyncAllResourceWithStreamingResponse:
    def __init__(self, all: AsyncAllResource) -> None:
        self._all = all

        self.delete = async_to_streamed_response_wrapper(
            all.delete,
        )
        self.status = async_to_streamed_response_wrapper(
            all.status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "TaskListResponse",
    "Data",
    "DataTask",
    "DataTaskBrowserConfiguration",
    "DataTaskBrowserConfigurationLiveView",
    "DataTaskBrowserConfigurationProxy",
    "DataTaskBrowserConfigurationProxyAnchorProxy",
    "DataTaskBrowserConfigurationProxyCustomProxy",
    "DataTaskBrowserConfigurationRecording",
    "DataTaskBrowserConfigurationTimeout",
]


class DataTaskBrowserConfigurationLiveView(BaseModel):
    """Configuration for live viewing the browser session."""

    read_only: Optional[bool] = None
    """Enable or disable read-only mode for live viewing. Defaults to `false`."""


class DataTaskBrowserConfigurationProxyAnchorProxy(BaseModel):
    active: bool

    city: Optional[str] = None
    """City name for precise geographic targeting.

    Supported for anchor_proxy only. Can only be used when region is also provided.
    """

    country_code: Optional[
        Literal[
            "af",
            "al",
            "dz",
            "ad",
            "ao",
            "as",
            "ag",
            "ar",
            "am",
            "aw",
            "au",
            "at",
            "az",
            "bs",
            "bh",
            "bb",
            "by",
            "be",
            "bz",
            "bj",
            "bm",
            "bo",
            "ba",
            "br",
            "bg",
            "bf",
            "cm",
            "ca",
            "cv",
            "td",
            "cl",
            "co",
            "cg",
            "cr",
            "ci",
            "hr",
            "cu",
            "cy",
            "cz",
            "dk",
            "dm",
            "do",
            "ec",
            "eg",
            "sv",
            "ee",
            "et",
            "fo",
            "fi",
            "fr",
            "gf",
            "pf",
            "ga",
            "gm",
            "ge",
            "de",
            "gh",
            "gi",
            "gr",
            "gd",
            "gp",
            "gt",
            "gg",
            "gn",
            "gw",
            "gy",
            "ht",
            "hn",
            "hu",
            "is",
            "in",
            "ir",
            "iq",
            "ie",
            "il",
            "it",
            "jm",
            "jp",
            "jo",
            "kz",
            "kw",
            "kg",
            "lv",
            "lb",
            "ly",
            "li",
            "lt",
            "lu",
            "mk",
            "ml",
            "mt",
            "mq",
            "mr",
            "mx",
            "md",
            "mc",
            "me",
            "ma",
            "nl",
            "nz",
            "ni",
            "ng",
            "no",
            "pk",
            "pa",
            "py",
            "pe",
            "ph",
            "pl",
            "pt",
            "pr",
            "qa",
            "ro",
            "lc",
            "sm",
            "sa",
            "sn",
            "rs",
            "sc",
            "sl",
            "sk",
            "si",
            "so",
            "za",
            "kr",
            "es",
            "sr",
            "se",
            "ch",
            "sy",
            "st",
            "tw",
            "tj",
            "tg",
            "tt",
            "tn",
            "tr",
            "tc",
            "ua",
            "ae",
            "us",
            "uy",
            "uz",
            "ve",
            "ye",
            "bd",
            "bw",
            "bn",
            "bi",
            "kh",
            "cn",
            "dj",
            "gq",
            "sz",
            "fj",
            "hk",
            "id",
            "ke",
            "la",
            "ls",
            "lr",
            "mg",
            "mw",
            "my",
            "mv",
            "mn",
            "mz",
            "mm",
            "na",
            "np",
            "nc",
            "ne",
            "om",
            "pg",
            "ru",
            "rw",
            "ws",
            "sg",
            "ss",
            "lk",
            "sd",
            "tz",
            "th",
            "tl",
            "tm",
            "ug",
            "gb",
            "vu",
            "vn",
            "zm",
            "zw",
            "bt",
            "mu",
        ]
    ] = None
    """Supported country codes ISO 2 lowercase

    **On change make sure to update the Proxy type.**
    """

    region: Optional[str] = None
    """
    Region code for more specific geographic targeting. The city parameter can only
    be used when region is also provided.
    """

    type: Optional[Literal["anchor_proxy", "anchor_residential", "anchor_mobile", "anchor_gov"]] = None
    """**On change make sure to update the country_code.**"""


class DataTaskBrowserConfigurationProxyCustomProxy(BaseModel):
    active: bool

    password: str
    """Proxy password"""

    server: str
    """Proxy server address"""

    type: Literal["custom"]

    username: str
    """Proxy username"""


DataTaskBrowserConfigurationProxy: TypeAlias = Union[
    DataTaskBrowserConfigurationProxyAnchorProxy, DataTaskBrowserConfigurationProxyCustomProxy
]


class DataTaskBrowserConfigurationRecording(BaseModel):
    """Configuration for session recording."""

    active: Optional[bool] = None
    """Enable or disable video recording of the browser session. Defaults to `true`."""


class DataTaskBrowserConfigurationTimeout(BaseModel):
    """Timeout configurations for the browser session."""

    idle_timeout: Optional[int] = None
    """
    The amount of time (in minutes) the browser session waits for new connections
    after all others are closed before stopping. Defaults to `5`.
    """

    max_duration: Optional[int] = None
    """Maximum amount of time (in minutes) for the browser to run before terminating.

    Defaults to `20`.
    """


class DataTaskBrowserConfiguration(BaseModel):
    """Browser configuration for task execution"""

    initial_url: Optional[str] = None
    """The URL to navigate to when the browser session starts.

    If not provided, the browser will load an empty page.
    """

    live_view: Optional[DataTaskBrowserConfigurationLiveView] = None
    """Configuration for live viewing the browser session."""

    proxy: Optional[DataTaskBrowserConfigurationProxy] = None
    """Proxy Documentation available at [Proxy Documentation](/advanced/proxy)"""

    recording: Optional[DataTaskBrowserConfigurationRecording] = None
    """Configuration for session recording."""

    timeout: Optional[DataTaskBrowserConfigurationTimeout] = None
    """Timeout configurations for the browser session."""


class DataTask(BaseModel):
    id: str
    """Unique identifier for the task"""

    code: str
    """Base64 encoded task code"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Task creation timestamp"""

    deleted: bool
    """Whether the task is soft deleted"""

    language: Literal["typescript"]
    """Programming language for the task"""

    latest_version: str = FieldInfo(alias="latestVersion")
    """Latest version identifier (draft, latest, or version number)"""

    name: str
    """Task name (letters, numbers, hyphens, and underscores only)"""

    team_id: str = FieldInfo(alias="teamId")
    """Team identifier that owns this task"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Task last update timestamp"""

    browser_configuration: Optional[DataTaskBrowserConfiguration] = FieldInfo(
        alias="browserConfiguration", default=None
    )
    """Browser configuration for task execution"""

    description: Optional[str] = None
    """Optional description of the task"""


class Data(BaseModel):
    limit: int
    """Number of tasks per page"""

    page: int
    """Current page number"""

    tasks: List[DataTask]

    total: int
    """Total number of tasks"""


class TaskListResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["ExtensionUploadParams"]


class ExtensionUploadParams(TypedDict, total=False):
    file: Required[FileTypes]
    """ZIP file containing the browser extension"""

    name: Required[str]
    """User-friendly name for the extension (1-255 characters)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.sessions import clipboard_set_params
from ...types.sessions.clipboard_get_response import ClipboardGetResponse
from ...types.sessions.clipboard_set_response import ClipboardSetResponse

__all__ = ["ClipboardResource", "AsyncClipboardResource"]


class ClipboardResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClipboardResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return ClipboardResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClipboardResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return ClipboardResourceWithStreamingResponse(self)

    def get(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClipboardGetResponse:
        """
        Retrieves the current content of the clipboard

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v1/sessions/{session_id}/clipboard",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClipboardGetResponse,
        )

    def set(
        self,
        session_id: str,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClipboardSetResponse:
        """
        Sets the content of the clipboard

        Args:
          text: Text to set in the clipboard

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/clipboard",
            body=maybe_transform({"text": text}, clipboard_set_params.ClipboardSetParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClipboardSetResponse,
        )


class AsyncClipboardResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClipboardResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncClipboardResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClipboardResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncClipboardResourceWithStreamingResponse(self)

    async def get(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClipboardGetResponse:
        """
        Retrieves the current content of the clipboard

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v1/sessions/{session_id}/clipboard",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClipboardGetResponse,
        )

    async def set(
        self,
        session_id: str,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClipboardSetResponse:
        """
        Sets the content of the clipboard

        Args:
          text: Text to set in the clipboard

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/clipboard",
            body=await async_maybe_transform({"text": text}, clipboard_set_params.ClipboardSetParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClipboardSetResponse,
        )


class ClipboardResourceWithRawResponse:
    def __init__(self, clipboard: ClipboardResource) -> None:
        self._clipboard = clipboard

        self.get = to_raw_response_wrapper(
            clipboard.get,
        )
        self.set = to_raw_response_wrapper(
            clipboard.set,
        )


class AsyncClipboardResourceWithRawResponse:
    def __init__(self, clipboard: AsyncClipboardResource) -> None:
        self._clipboard = clipboard

        self.get = async_to_raw_response_wrapper(
            clipboard.get,
        )
        self.set = async_to_raw_response_wrapper(
            clipboard.set,
        )


class ClipboardResourceWithStreamingResponse:
    def __init__(self, clipboard: ClipboardResource) -> None:
        self._clipboard = clipboard

        self.get = to_streamed_response_wrapper(
            clipboard.get,
        )
        self.set = to_streamed_response_wrapper(
            clipboard.set,
        )


class AsyncClipboardResourceWithStreamingResponse:
    def __init__(self, clipboard: AsyncClipboardResource) -> None:
        self._clipboard = clipboard

        self.get = async_to_streamed_response_wrapper(
            clipboard.get,
        )
        self.set = async_to_streamed_response_wrapper(
            clipboard.set,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SessionCreateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the browser session"""

    cdp_url: Optional[str] = None
    """The CDP websocket connection string"""

    live_view_url: Optional[str] = None
    """The browser session live view url"""


class SessionCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ToolPerformWebTaskParams"]


class ToolPerformWebTaskParams(TypedDict, total=False):
    prompt: Required[str]
    """The task to be autonomously completed."""

    session_id: Annotated[str, PropertyInfo(alias="sessionId")]
    """
    An optional browser session identifier to reference an existing running browser
    sessions. When passed, the tool will be executed on the provided browser
    session.
    """

    agent: Literal["browser-use", "openai-cua"]
    """The AI agent to use for task completion. Defaults to browser-use."""

    detect_elements: bool
    """Enable element detection for better interaction accuracy.

    Improves the agent's ability to identify and interact with UI elements.
    """

    highlight_elements: bool
    """Whether to highlight elements during task execution for better visibility."""

    human_intervention: bool
    """Allow human intervention during task execution.

    When enabled, the agent can request human input for ambiguous situations.
    """

    max_steps: int
    """Maximum number of steps the agent can take to complete the task.

    Defaults to 200.
    """

    model: str
    """The specific model to use for task completion.

    see our [models](/agentic-browser-control/ai-task-completion#available-models)
    page for more information.
    """

    output_schema: object
    """JSON Schema defining the expected structure of the output data."""

    provider: Literal["openai", "gemini", "groq", "azure", "xai"]
    """The AI provider to use for task completion."""

    secret_values: Dict[str, str]
    """Secret values to pass to the agent for secure credential handling.

    Keys and values are passed as environment variables to the agent.
    """

    url: str
    """The URL of the webpage.

    If not provided, the tool will use the current page in the session.
    """

    directly_open_url: bool
    """If true, the tool will directly open the URL in the browser."""


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["MouseClickParams"]


class MouseClickParams(TypedDict, total=False):
    button: Literal["left", "middle", "right"]
    """Mouse button to use"""

    index: float
    """
    If a selector was passed and multiple elements match the selector, the index of
    the element in the list (0-based). Defaults to 0.
    """

    selector: str
    """A valid CSS selector for the requested element"""

    selector_timeout_ms: Annotated[float, PropertyInfo(alias="timeout")]
    """
    If a selector was passed, timeout in ms for waiting for the DOM element to be
    selected. Defaults to 5000 (5 seconds).
    """

    x: float
    """X coordinate"""

    y: float
    """Y coordinate"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["EventSignalParams"]


class EventSignalParams(TypedDict, total=False):
    data: Required[Dict[str, object]]
    """Event data to be passed to waiting clients"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ToolFetchWebpageParams"]


class ToolFetchWebpageParams(TypedDict, total=False):
    session_id: Annotated[str, PropertyInfo(alias="sessionId")]
    """
    An optional browser session identifier to reference an existing running browser
    session. If provided, the tool will execute within that browser session.
    """

    format: Literal["html", "markdown"]
    """The output format of the content."""

    new_page: bool
    """Whether to create a new page for the content."""

    page_index: int
    """The index of the page to fetch content from. **Overides new_page**."""

    return_partial_on_timeout: bool
    """
    Whether to return partial content if the content is not loaded within the 20
    seconds.
    """

    url: str
    """The URL of the webpage to fetch content from.

    When left empty, the current webpage is used.
    """

    wait: int
    """The time to wait for **dynamic** content to load in **milliseconds**."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ProfileCreateParams"]


class ProfileCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name of the profile."""

    dedicated_sticky_ip: bool
    """Whether to use a dedicated sticky IP for this profile. Defaults to false."""

    description: str
    """A description of the profile."""

    session_id: str
    """The browser session ID is required if the source is set to `session`.

    The browser session must be running, and the profile will be stored once the
    browser session terminates.
    """

    source: Literal["session"]
    """The source of the profile data. currently only `session` is supported."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..shared.success_response import SuccessResponse

__all__ = ["RecordingPauseResponse"]


class RecordingPauseResponse(BaseModel):
    data: Optional[SuccessResponse] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["RecordingListResponse", "Data", "DataItem"]


class DataItem(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the recording"""

    created_at: Optional[datetime] = None
    """Timestamp when the recording was created"""

    duration: Optional[str] = None
    """Duration of the recording"""

    file_link: Optional[str] = None
    """URL to access the recording file"""

    is_primary: Optional[bool] = None
    """Indicates if this is the primary recording"""

    size: Optional[float] = None
    """Size of the recording file in bytes"""

    suggested_file_name: Optional[str] = None
    """Suggested filename for the recording"""


class Data(BaseModel):
    count: Optional[int] = None
    """Total number of video recordings"""

    items: Optional[List[DataItem]] = None


class RecordingListResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["KeyboardShortcutParams"]


class KeyboardShortcutParams(TypedDict, total=False):
    keys: Required[SequenceNotStr[str]]
    """Array of keys to press simultaneously"""

    hold_time: Annotated[int, PropertyInfo(alias="holdTime")]
    """Time to hold the keys down in milliseconds"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ..types import event_signal_params, event_wait_for_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.event_wait_for_response import EventWaitForResponse
from ..types.shared.success_response import SuccessResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def signal(
        self,
        event_name: str,
        *,
        data: Dict[str, object],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuccessResponse:
        """
        Signals an event with associated data, unblocking any clients waiting for this
        event. This enables coordination between different browser sessions, workflows,
        or external processes.

        Args:
          data: Event data to be passed to waiting clients

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_name:
            raise ValueError(f"Expected a non-empty value for `event_name` but received {event_name!r}")
        return self._post(
            f"/v1/events/{event_name}",
            body=maybe_transform({"data": data}, event_signal_params.EventSignalParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponse,
        )

    def wait_for(
        self,
        event_name: str,
        *,
        timeout_ms: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventWaitForResponse:
        """
        Waits for a specific event to be signaled by another process, workflow, or
        session. This endpoint blocks until the event is signaled or the timeout is
        reached. Useful for coordinating between multiple browser sessions or workflows.

        Args:
          timeout_ms: Timeout in milliseconds to wait for the event. Defaults to 60000ms (1 minute).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_name:
            raise ValueError(f"Expected a non-empty value for `event_name` but received {event_name!r}")
        return self._post(
            f"/v1/events/{event_name}/wait",
            body=maybe_transform({"timeout_ms": timeout_ms}, event_wait_for_params.EventWaitForParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventWaitForResponse,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    async def signal(
        self,
        event_name: str,
        *,
        data: Dict[str, object],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuccessResponse:
        """
        Signals an event with associated data, unblocking any clients waiting for this
        event. This enables coordination between different browser sessions, workflows,
        or external processes.

        Args:
          data: Event data to be passed to waiting clients

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_name:
            raise ValueError(f"Expected a non-empty value for `event_name` but received {event_name!r}")
        return await self._post(
            f"/v1/events/{event_name}",
            body=await async_maybe_transform({"data": data}, event_signal_params.EventSignalParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponse,
        )

    async def wait_for(
        self,
        event_name: str,
        *,
        timeout_ms: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventWaitForResponse:
        """
        Waits for a specific event to be signaled by another process, workflow, or
        session. This endpoint blocks until the event is signaled or the timeout is
        reached. Useful for coordinating between multiple browser sessions or workflows.

        Args:
          timeout_ms: Timeout in milliseconds to wait for the event. Defaults to 60000ms (1 minute).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_name:
            raise ValueError(f"Expected a non-empty value for `event_name` but received {event_name!r}")
        return await self._post(
            f"/v1/events/{event_name}/wait",
            body=await async_maybe_transform({"timeout_ms": timeout_ms}, event_wait_for_params.EventWaitForParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventWaitForResponse,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.signal = to_raw_response_wrapper(
            events.signal,
        )
        self.wait_for = to_raw_response_wrapper(
            events.wait_for,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.signal = async_to_raw_response_wrapper(
            events.signal,
        )
        self.wait_for = async_to_raw_response_wrapper(
            events.wait_for,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.signal = to_streamed_response_wrapper(
            events.signal,
        )
        self.wait_for = to_streamed_response_wrapper(
            events.wait_for,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.signal = async_to_streamed_response_wrapper(
            events.signal,
        )
        self.wait_for = async_to_streamed_response_wrapper(
            events.wait_for,
        )
